import click
import webbrowser
import json
import webbrowser
import threading
import readline # is required to lift the limitation of input() in macOS of 1024 characters (View https://stackoverflow.com/a/53871077)
from rich.console import Console
from rich.text import Text
from aichor_cli import cli
from aichor_cli.client.client import Client
from aichor_cli.client.experiment import Experiment
from aichor_cli.client.project import Project
from aichor_cli.server_auth import run_server
from aichor_cli.table_utils import display_projects, display_experiments,set_project
from aichor_cli.version import __version__
from aichor_cli.config import get_config,get_config_paths

config=get_config()
config_path,config_dir= get_config_paths()
client = Client(config_path)
project = Project(client) 
experiment = Experiment(client,project)
console = Console()

@cli.command()
def login():
    """Login to access AIchor """
    # Start the web server
    click.echo("Starting the web server...")
    server_thread = threading.Thread(target=run_server)
    server_thread.start() 
    jaqen_service = config.get('jaqenService')
    base_url = config.get('baseURL')
    stage = config.get('stage')
    url = f"{jaqen_service}{base_url}/{stage}access-token-display"
    # Redirect to the login page in the web browser
    click.echo("Redirecting to the login page in your web browser...")
    webbrowser.open_new(url)

@cli.command()
def configure():
    """Configure Token"""
    console.print("\n[bold cyan]Enter Your Token[/]\n", style="grey69 italic")
    token = input()
    config['credentials']['token'] = token
    with open(config_path, "w") as config_file:
        json.dump(config, config_file)
    console.print("Token saved successfully.", style="bold green")

@cli.command()
@click.argument('subject', type=click.Choice(['project', 'projects', 'experiments','experiment']))
@click.option('--project-id',help='ID of the project')
@click.option('--experiment-id',help='ID of the experiment',default=None)
def get(subject, project_id,experiment_id):
    """Display resources"""
    if subject in ['project', 'projects'] and experiment_id is not None:
        raise click.UsageError("--experiment-id option is not allowed with 'get project' or 'get projects' commands.")
    if subject in ['project', 'projects']:   
        display_projects(project_id)        
    elif subject in ['experiment','experiments']: 
        display_experiments(project_id,experiment_id)
    else:
        click.echo("Invalid subject")

@cli.command()
@click.argument('subject', type=click.Choice(['project','projects']))
@click.option('--project-id',required='True',help='ID of the project to set as default')
def set(subject, project_id):
    "Set Default Project"
    if subject in ['project', 'projects']:
        set_project(project_id)
    else:
        click.echo("Invalid subject")

@cli.command()
@click.argument('subject', type=click.Choice(['experiment', 'experiments']))
@click.option('--project-id', help='ID of the project to resubmit its experiments')
@click.option('--experiment-id', help='ID of the experiment')
def resubmit(subject, project_id, experiment_id):
    """Resubmit an experiment"""
    if subject in ['experiment', 'experiments']:
        experiment.resubmit_experiments(project_id, experiment_id)
    else:
        console.print("Invalid subject")
        
@cli.command()
@click.argument('subject', type=click.Choice(['experiment','experiments']))
@click.option('--project-name', required=True, help='Name of the project to submit its experiment')
@click.option('--repo-dir', required=True, help='The repo cloned by the user')
@click.option('--message', required=True,help='Commit message')
def submit(subject,project_name,repo_dir,message):
    """Submit an experiment manually triggered"""
    console.print("Manual trigger experiment...",style="green")
    if subject in ['experiment','experiments']:
        experiment.submit_experiment(project_name,repo_dir,message)
      
if __name__ == '__main__':
    cli()

