import json
import requests
from typing import Dict, List, Optional
from rich.console import Console
from aichor_cli.config import get_config_paths
from aichor_cli.client.client import Client

console = Console()
config_path, config_dir = get_config_paths()
class Project:
    def __init__(self,client: Client):
        self.client = client

    def get_api_endpoint(self) -> str:
        roles = self.client.get_user_roles()
        if "admin" in roles or "super admin" in roles:
            return '/projects'
        else:
            return '/user/projects'   
        
    def get_project_info(self,project_id):
        try:
            endpoint = self.get_api_endpoint()
            response_json = self.client.handle_request(endpoint,'get')
            project = next((project for project in response_json["result"] if project["projectId"] == project_id), None)
            return project   
        except requests.exceptions.RequestException as e:
            print("Error retrieving project data:", e)

    def list_projects(self) -> List[Dict]:
        endpoint = self.get_api_endpoint()
        response_json = self.client.handle_request(endpoint,'get')
        projects_info = []
        if response_json.get('result'):
            default_project_name = self.get_default_project_name()
            for index, element in enumerate(response_json["result"]):
                project_info = {
                    "id": element["projectId"],
                    "index": index + 1,
                    "name": element["name"],
                    "createdOn": element["createdOn"],
                    "status": element["status"],
                    "totalCost": element["billingCost"]["totalCost"],
                    "default": "⭐" if element["name"] == default_project_name else ""
                }
                projects_info.append(project_info)
        return projects_info

    def get_project_id(self, project_name: str) -> str:
        endpoint = self.get_api_endpoint()
        data = self.client.handle_request(endpoint,'get')
        if data.get('result'):
            for project in data['result']:
                if project['name'] == project_name:
                    return project['projectId']
        return ""

    def get_project_name(self, project_id: str) -> str:
        endpoint = self.get_api_endpoint()
        data = self.client.handle_request(endpoint,'get')
        if data.get('result'):
            for project in data['result']:
                if project['projectId'] == project_id:
                    return project['name']
        return ""

    def get_default_project_name(self) -> Optional[str]:
        try:
            with open(config_path, "r") as config_file:
                config = json.load(config_file)
                if 'context' in config and 'projectName' in config['context']:
                    return config['context']['projectName']
        except (FileNotFoundError, json.JSONDecodeError) as e:
            print("Error reading config file:", e)
        return None
    
    def load_default_project(self,config_path) -> str:
        with open(config_path, "r") as config_file:
            config = json.load(config_file)
            project_id = config.get("context", {}).get("projectID")
        if not project_id:
            console.print("[red][bold]Project id is required: Please set a project or add missing argument --project-id[/bold][/red]")
            return None
        return project_id