import json
from http.server import BaseHTTPRequestHandler, HTTPServer
from rich.console import Console
from aichor_cli.config import get_config,get_config_paths

"""
This script sets up and runs a basic HTTP server to receive and save access tokens.
"""
config_path,config_dir=get_config_paths()
config=get_config()
console = Console()

#Flag to control the server state
server_running = True
class RequestHandler(BaseHTTPRequestHandler):
    def _set_headers(self):
        self.send_response(200)
        self.send_header('Content-type', 'application/json')
        self.send_header('Access-Control-Allow-Origin', '*')
        self.send_header('Access-Control-Allow-Methods', 'POST, OPTIONS')
        self.send_header('Access-Control-Allow-Headers', 'Content-Type')
        self.end_headers()

    def do_OPTIONS(self):
        self._set_headers()

    def do_POST(self):
        global server_running
        content_length = int(self.headers['Content-Length'])
        post_data = self.rfile.read(content_length)
        post_data_str = post_data.decode('utf-8')
        # Retrieve token from POST data
        token_data = json.loads(post_data_str)
        access_token = token_data.get("accessToken")
        if access_token:
            config['credentials']['token'] = access_token
            with open(config_path, "w") as config_file:
                json.dump(config, config_file)
            response_data = {
            'status': 'success',
            'message': 'Token received and saved successfully'
        }
            console.print("[magenta][bold]Token copied and saved successfully![/bold][/magenta]")
            server_running = False
        else:
            response_data = {
            'status': 'error',
            'message': 'No access token provided'
        }
            console.print("[red][bold]No token provided![/bold][/red]")
        self._set_headers()
        self.wfile.write(json.dumps(response_data).encode('utf-8'))

def run_server(port=4379):
    global server_running
    server_address = ('localhost', port)
    httpd = HTTPServer(server_address, RequestHandler)
    print(f'Starting server on localhost:{port}...')
    while server_running:
        httpd.handle_request()
