import json
import os
import click
import os
import zipfile
from rich.console import Console
from aichor_cli import cli
from aichor_cli.config import get_config_paths

console = Console()
config_path, config_dir = get_config_paths()
 
def set_default_project_in_config(config_path, default_project_name, default_project_id):
    try:
        with open(config_path, "r") as config_file:
            config = json.load(config_file)
    except FileNotFoundError:
        config = {}
    if 'context' not in config:
        config['context'] = {}
    config['context']['projectName'] = default_project_name
    config['context']['projectID'] = default_project_id
    with open(config_path, "w") as config_file:
        json.dump(config, config_file, indent=4)
  
def zip_directory(directory_path, zip_path):
    """
    Zips the specified directory and saves it as a zip file.

    :param directory_path: Path to the directory to be zipped.
    :param zip_path: Path where the output zip file should be saved.
    """
    with zipfile.ZipFile(zip_path, 'w') as zipf:
        for root, dirs, files in os.walk(directory_path):
            for file in files:
                file_path = os.path.join(root, file)
                arcname = os.path.relpath(file_path, start=directory_path)
                zipf.write(file_path, arcname)
    console.print(f'Directory {directory_path} zipped successfully as {zip_path}', style="green")

def alias(*aliases):
    def decorator(f):
        click_command = click.command(name=f.__name__)(f)
        for alias in aliases:
            cmd = click.command(name=alias, cls=click.Command)(f)
            cmd.params = click_command.params
            cmd.short_help = click_command.short_help
            cmd.help = click_command.help
            cli.add_command(cmd)
        return f
    return decorator

def obscure(text, flank_length=4):
    """Obscures text by keeping flank_length characters at each end and replacing the rest with *'s."""
    if len(text) < flank_length * 2:
        return text
    obscured_text = text[:flank_length] + '*' * 4 + text[-flank_length:]
    return obscured_text